#include <msp430x14x.h>
#include "help.c"
#include "fonts.h"
#include "pins.h"
#include "glcd.c"
#include "lcd.c"
#include "tmp100.c"
#include "usart.c"

char temp_str[]="0000";
char uhr_str[]="00";
char hundertstelsekunden=0, sekunden=0, minuten=8, stunden=21, tage=0;
char GotUSARTChar;

int main(void)
{
  WDTCTL = WDTPW + WDTHOLD;  // watchdog aus

  DATADIR=0xFF;           // DATAPORT = output (8 datenleitungen zum GLCD)
  CSDIR=0xFF;             // CSPORT = output (CS1 und CS2 zum GLCD)
  CONFIGDIR=0xEF;         // CONFIGPORT = output (RW, DI und E zum GLCD), input (100Hz-erkennung an pin 16)
  LCDDATADIR=0xFF;        // LCDDATAPORT = output (8 datenleitungen zum LCD)
  LCDCONFIGDIR=0xFF;      // LCDDATADIR = output (RS, RW und E zum LCD)

  P3SEL=0xF0;             // P3.4 bis P3.7 als USART0 bzw. USART1 TXD/RXD benutzen

  _BIS_SR(GIE);           // allgemeiner interrupt-enable
  P1IE=0x10;              // pin 16 interrupt-faehig machen (wg. 100Hz-erkennung)
  P1IES=0x10;             // interrupt auf pin 16 ausloesen bei high->low - uebergang

  External_OSC_on();
  InitUSART(1,0,19200,0,0,0);

  GLCDInit('b');

  for(;;)
  {
    GLCDSetRow(0, 'b');

    GLCDOuts("T1:", 0, 7, "6x8");
    GLCDOuts("C", 42, 7, "6x8");
    GLCDOuts("---------------------", 0, 6, "6x8");
    GLCDOuts("T2:", 72, 7, "6x8");
    GLCDOuts("C", 114, 7, "6x8");
    ReadTMP100(temp_str, 0x49);
    GLCDOuts(temp_str, 18, 7, "6x8");
    ReadTMP100(temp_str, 0x4A);
    GLCDOuts(temp_str, 90, 7, "6x8");
    
    IntToString(uhr_str, stunden, 2, 0);
    GLCDOuts(uhr_str, 15, 3, "12x16");
    GLCDOuts(":", 39, 3, "12x16");
    IntToString(uhr_str, minuten, 2, 0);
    GLCDOuts(uhr_str, 50, 3, "12x16");
    IntToString(uhr_str, sekunden, 2, 0);
    GLCDOuts(":", 75, 3, "12x16");
    GLCDOuts(uhr_str, 86, 3, "12x16");
  }
}

#pragma vector=UART0RX_VECTOR
__interrupt void usart0_rx (void)
{
  if (RXBUF0!='s'&&RXBUF0!='m') TXBUF0=RXBUF0;
  if (GotUSARTChar==2)
  {
    stunden+=RXBUF0-48;
    GotUSARTChar=0;
  }
  if (GotUSARTChar==1)
  {
    stunden=10*(RXBUF0-48);
    GotUSARTChar=2;
  }
  if (GotUSARTChar==5)
  {
    minuten+=RXBUF0-48;
    GotUSARTChar=0;
  }
  if (GotUSARTChar==4)
  {
    minuten=10*(RXBUF0-48);
    GotUSARTChar=5;
  }
  if (RXBUF0=='s')
  {
    SendUSART0c(13);
    SendUSART0("Bitte Stunden eingeben: ");
    GotUSARTChar=1;
  }
  if (RXBUF0=='m')
  {
    SendUSART0c(13);
    SendUSART0("Bitte Minuten eingeben: ");
    GotUSARTChar=4;
  }
}

#pragma vector=PORT1_VECTOR
__interrupt void Port1_Interrupt (void)
{
  if (P1IFG&0x10)         // 0x10 ist pin 16
  {
  if (!((++hundertstelsekunden)%50)) 
    {                     // experimentell: uhrzeit ausgeben ...
      hundertstelsekunden = 0;
      if (!((++sekunden)%60))
      {
        sekunden = 0;
        if (!((++minuten)%60))
        {
          minuten = 0;
          if (!((++stunden)%24))
          {
            stunden = 0;
            tage++;
          }
        }
      }
    }
  }
  P1IFG&=~0x10;
}
